#!/usr/bin/env python

# We will use the inkex module with the predefined Effect base class.
import inkex
# The simplestyle module provides functions for style parsing.
from simplestyle import *
from math import tan, radians

def points_to_svgd(p, close=True):
	""" convert list of points (x,y) pairs
		into a closed SVG path list
	"""
	f = p[0]
	p = p[1:]
	svgd = 'M%.4f,%.4f' % f
	for x in p:
		svgd += 'L%.4f,%.4f' % x
	if close:
		svgd += 'z'
	return svgd
	
def create_path(points, color, group):
	# Create path
	path = points_to_svgd( points )
	# define style using basic dictionary
	style = { 'stroke': color, 'fill': 'none', 'stroke-width': 1.0 }
	# convert style into svg form (see import at top of file)
	mypath_attribs = { 'style': formatStyle(style), 'd': path }
	# add path to scene (Create SVG Path under group)
	inkex.etree.SubElement(group, inkex.addNS('path','svg'), mypath_attribs )

class MiuraEffect(inkex.Effect):
	"""
	Miura pattern effect extension.
	Creates a new layer with the miura pattern on it.
	"""
	def __init__(self):
		"""
		Constructor.
		Defines the options of the script.
		"""
		# Call the base class constructor.
		inkex.Effect.__init__(self)

		# Define string options "--what" with "-w" shortcut and default value "World".
		self.OptionParser.add_option('--no_x', action = 'store',
		  type = 'int', dest = 'no_x', default = '2',
		  help = 'How many modules do you want horizontally?')
		self.OptionParser.add_option('--no_y', action = 'store',
		  type = 'int', dest = 'no_y', default = '2',
		  help = 'How many modules do you want vertically?')
		self.OptionParser.add_option('--angle', action = 'store',
		  type = 'int', dest = 'angle', default = '20',
		  help = 'Which angle should the pattern have?')
		self.OptionParser.add_option('--height_cell', action = 'store',
		  type = 'int', dest = 'height_cell', default = '10',
		  help = 'How high should each cell be?')
		self.OptionParser.add_option('--width_cell', action = 'store',
		  type = 'int', dest = 'width_cell', default = '10',
		  help = 'How wide should each cell be?')

	def effect(self):
		"""
		Effect behaviour.
		"""
		# Get script's option values.
		no_x = self.options.no_x
		no_y = self.options.no_y
		a = self.options.angle
		h = self.options.height_cell
		w = self.options.width_cell
		
		# calculate helper var
		# y-offset
		y_o = w*tan(radians(a))
		
		# This finds center of current view in inkscape
		t = 'translate(%s,%s)' % (self.view_center[0], self.view_center[1] )
		# Make a name for the group in which the paths will be added and set position
		g_attribs = { inkex.addNS('label','inkscape'): 'Miura Pattern',
					  'transform': t }
		# add the group to the document's current layer
		topgroup = inkex.etree.SubElement(self.current_layer, 'g', g_attribs )
		
		# Create outline
		y1 = 0
		y2 = y_o
		for x in range(no_x):
			create_path([(x*w,y1),((x+1)*w,y2)], "black", topgroup);
			create_path([(x*w,y1+no_y*h),((x+1)*w,y2+no_y*h)], "black", topgroup);
			y1, y2 = y2, y1
		for y in range(no_y):
			create_path([(0,y*h),(0,(y+1)*h)], "black", topgroup);
			if no_x % 2 == 0: #even number of cells
				create_path([(no_x*w,y*h),(no_x*w,(y+1)*h)], "black", topgroup);
			else:
				create_path([(no_x*w,y*h+y_o),(no_x*w,(y+1)*h+y_o)], "black", topgroup);
		
		# Create horizontal lines
		c = "red"
		for y in range(1,no_y):
			y1 = 0
			y2 = y_o
			for x in range(no_x):
				create_path([(x*w,y1+y*h),((x+1)*w,y2+y*h)], c, topgroup);
				y1, y2 = y2, y1
			c = "blue" if c=="red" else "red"
			
		# Create vertical lines
		c_s = "blue"
		y1 = y_o
		y2 = 0
		for x in range(1, no_x):
			c = c_s
			for y in range(no_y):
				create_path([(x*w,y1+y*h),(x*w,y1+(y+1)*h)], c, topgroup);
				c = "blue" if c=="red" else "red"
			y1, y2 = y2, y1
			c_s = "blue" if c_s=="red" else "red"
			

# Create effect instance and apply it.
effect = MiuraEffect()
effect.affect()
